/*
 *  TemperatureSensor.java
 *
 *  Modified:  21 Sep 99 by Dean Addison (GSI Canada Ltd.)
 *  	- changed to force a read from the Thermochron iButton (DS1921)
 */

/*  Copyright (C) 1998 Dallas Semiconductor Corporation.
 *  All rights Reserved. Printed in U.S.A.
 *  This software is protected by copyright laws of
 *  the United States and of foreign countries.
 *  This material may also be protected by patent laws of the United States
 *  and of foreign countries.
 *  This software is furnished under a license agreement and/or a
 *  nondisclosure agreement and may only be used or copied in accordance
 *  with the terms of those agreements.
 *  The mere transfer of this software does not imply any licenses
 *  of trade secrets, proprietary technology, copyrights, patents,
 *  trademarks, maskwork rights, or any other form of intellectual
 *  property whatsoever. Dallas Semiconductor retains all ownership rights.
 */

import com.ibutton.*;
import com.dalsemi.comm.*;

/**
 *  This class encapsulates the behavior of the often immitated, never
 *  duplicated DS1921.
 */
public class TemperatureSensor
{
  // DS1921 specific commands
  public static final int CONVERT_TEMPERATURE	= 0x44;
  public static final int MATCH_ROM					= 0x55;
  public static final int READ_MEMORY				= 0xF0;  // DMA--changed.

  public static final int DS1921_FAMILY_CODE		= 0x21;  // DMA--changed.
  // Last read temperature in degrees C
  private int temperature;
  private Access ds1921;

  /*
  * Holds the 8 byte rom Id
  */
  int[] romId;
  int[] block = new int[13];  // DMA--now 13 instead of 12.

  /**
  * Construct a new instance of TemperatureSensor with the default bus speed.
  */
  public TemperatureSensor()
  {
    try
    {
      ds1921  = new Access();
      ds1921.SetAdapterType(SerialPort.SERIAL0, "port0");
      findSensor();
    }
    catch(Exception e){}
  }
  /** Find a temp sensor.
    */
  public boolean findSensor()
  {
    boolean foundDS1921 = false;
    romId = null;

    // find the first temp sensor
    if(ds1921.iBReset())
    {
      if(ds1921.iBFirst())
      {
        romId   = ds1921.iBROMData();

        do
        {
          if(romId[0] == DS1921_FAMILY_CODE)
          {
            foundDS1921 = true;
            break;
          }
        }while(ds1921.iBNext());
      }
    }

    return foundDS1921;
  }
  /** Send the ConvertT command to the device
    */
  public void startConversion()
  {
    if(ds1921.iBReset())
    {
      block[0] = MATCH_ROM;
      // copy serial number
      System.arraycopy(romId, 0, block, 1, romId.length);
		
      block[9] = CONVERT_TEMPERATURE;

      ds1921.iBDataBlock(block, 0, 10);
      com.dalsemi.jellybeans.Control.delay(100);// TINI Specific
    }
  }
  /** Check the ready status.
    * Get the temperature (less precise version -
    *                      see 1921/1921 docs for more precise conversion)
    */
  public boolean isReady()
  {
    // DS1921 will return a 0 if it's still busy
    boolean done = ds1921.iBDataBit();

    // If the temperature conversion is finished read the new temperature
    if(done)
    {
      if(ds1921.iBReset())
      {
		  block[0]  = MATCH_ROM;
        // copy serial number
        System.arraycopy(romId, 0, block, 1, romId.length);
        block[9]  = READ_MEMORY;  //  DMA--Read memory not scratchpad.
        // 0x0211 for temperature data
        block[10] = 0x11;  //  DMA--Read from address 0x0211.
		  block[11] = 0x02;  //  DMA
		  block[12] = 0xFF;  //  DMA--Tell iBDataBlock to put the data in this element.
        // send it off
        if(!ds1921.iBDataBlock(block, 0, 13)) {
			  System.out.println(ds1921.iBFailReason());  //  DMA--Added.
		  }

        // read one byte only
        temperature = block[12];  //  DMA--This is temperature.

        ds1921.iBReset();  //  DMA--Added.

      }
      else
        return false;
    }
    else
      com.dalsemi.jellybeans.Control.delay(100);// TINI Specific

    return done;
  }
  /** Return serial number.
    */
  public int[] getROMId()
  {
    return romId;
  }
  /** Return the exact temperature.
    */
  public int getTemperatureCelsius()
  {
    // lsb is 0.5 degree -> truncated to integer
    return (temperature >> 1) - 40;  //  DMA--changed.  () required.
  }
  /** Return the temperature converted to fahrenheit.
    */
  public int getTemperatureFahrenheit()
  {
    return temperature*9/10 - 40;  //  DMA-changed.
  }
}
